==> crawler_rate_limit-1.0.0.tar.gz: crawler_rate_limit/src/RateLimitManager.php <==
<?php

namespace Drupal\crawler_rate_limit;

use Drupal\Core\Site\Settings;
use Drupal\redis\ClientFactory;
use Jaybizzle\CrawlerDetect\CrawlerDetect;
use RateLimit\Exception\LimitExceeded;
use RateLimit\Rate;
use RateLimit\RedisRateLimiter;
use Symfony\Component\HttpFoundation\Request;

/**
 * Rate Limit Manager.
 *
 * @package Drupal\crawler_rate_limit
 */
class RateLimitManager implements RateLimitManagerInterface {

  /**
   * Number of operations that is allowed in the given interval.
   *
   * @var int
   */
  private $operations;

  /**
   * Interval in seconds.
   *
   * @var int
   */
  private $interval;

  /**
   * Flag indicating whether rate limiter is enabled or not.
   *
   * @var bool
   */
  private $enabled;

  /**
   * Redis client.
   *
   * @var \Redis
   */
  private $redis;

  /**
   * CrawlerDetect. Detects bots/crawlers/spiders via the user agent.
   *
   * @var \Jaybizzle\CrawlerDetect\CrawlerDetect
   */
  private $crawlerDetect;

  /**
   * Class constructor.
   *
   * @param \Drupal\redis\ClientFactory $factory
   *   Redis client factory.
   * @param \Jaybizzle\CrawlerDetect\CrawlerDetect $crawlerDetect
   *   CrawlerDetect object.
   */
  public function __construct(ClientFactory $factory, CrawlerDetect $crawlerDetect) {
    $this->redis = $factory->getClient();
    $this->crawlerDetect = $crawlerDetect;

    $settings = Settings::get('crawler_rate_limit.settings', []);
    $settings += [
      'operations' => 100,
      'interval' => 60,
      'enabled' => TRUE,
    ];
    $this->operations = $settings['operations'];
    $this->interval = $settings['interval'];
    $this->enabled = $settings['enabled'];
  }

  /**
   * {@inheritdoc}
   */
  public function limit(Request $request) {
    if (!$this->isEnabled()) {
      return FALSE;
    }

    if (!$this->crawlerDetect->isCrawler()) {
      return FALSE;
    }

    $rate_limiter = new RedisRateLimiter($this->redis, 'ratelimit:');
    try {
      $rate_limiter->limit($this->crawlerDetect->getMatches(), Rate::custom($this->operations, $this->interval));
    }
    catch (LimitExceeded $exception) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled() {
    return $this->enabled;
  }

  /**
   * {@inheritdoc}
   */
  public function retryAfter() {
    return $this->interval;
  }

}

==> crawler_rate_limit-2.0.0.tar.gz: crawler_rate_limit/src/RateLimitManager.php <==
<?php

namespace Drupal\crawler_rate_limit;

use Drupal\Core\Site\Settings;
use Drupal\redis\ClientFactory;
use Jaybizzle\CrawlerDetect\CrawlerDetect;
use RateLimit\Exception\LimitExceeded;
use RateLimit\Rate;
use RateLimit\RedisRateLimiter;
use Symfony\Component\HttpFoundation\Request;

/**
 * Rate Limit Manager.
 *
 * @package Drupal\crawler_rate_limit
 */
class RateLimitManager implements RateLimitManagerInterface {

  /**
   * Number of operations that is allowed in the given interval.
   *
   * @var int
   */
  private $operations;

  /**
   * Interval in seconds.
   *
   * @var int
   */
  private $interval;

  /**
   * Flag indicating whether rate limiter is enabled or not.
   *
   * @var bool
   */
  private $enabled;

  /**
   * Redis client.
   *
   * @var \Redis
   */
  private $redis;

  /**
   * CrawlerDetect. Detects bots/crawlers/spiders via the user agent.
   *
   * @var \Jaybizzle\CrawlerDetect\CrawlerDetect
   */
  private $crawlerDetect;

  /**
   * Class constructor.
   *
   * @param \Drupal\redis\ClientFactory $factory
   *   Redis client factory.
   * @param \Jaybizzle\CrawlerDetect\CrawlerDetect $crawlerDetect
   *   CrawlerDetect object.
   */
  public function __construct(ClientFactory $factory, CrawlerDetect $crawlerDetect) {
    $this->redis = $factory->getClient();
    $this->crawlerDetect = $crawlerDetect;

    $settings = Settings::get('crawler_rate_limit.settings', []);
    $settings += [
      'operations' => 100,
      'interval' => 60,
      'enabled' => TRUE,
    ];
    $this->operations = $settings['operations'];
    $this->interval = $settings['interval'];
    $this->enabled = $settings['enabled'];
  }

  /**
   * {@inheritdoc}
   */
  public function limit(Request $request) {
    if (!$this->isEnabled()) {
      return FALSE;
    }

    if (!$this->crawlerDetect->isCrawler()) {
      return FALSE;
    }

    $rate_limiter = new RedisRateLimiter(Rate::custom($this->operations, $this->interval), $this->redis, 'ratelimit:');
    try {
      $rate_limiter->limit($this->crawlerDetect->getMatches());
    }
    catch (LimitExceeded $exception) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled() {
    return $this->enabled;
  }

  /**
   * {@inheritdoc}
   */
  public function retryAfter() {
    return $this->interval;
  }

}
